# Generated by Django 4.1.8 on 2023-05-23 08:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("agenda", "0005_building_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="calendaritem",
            name="is_deletable",
            field=models.BooleanField(default=True, verbose_name="Is Deletable"),
        ),
    ]
