# Generated by Django 5.0.6 on 2024-08-06 07:28

import wbcore.contrib.example_app.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("example_app", "0004_alter_player_transfer_value"),
    ]

    operations = [
        migrations.AddField(
            model_name="sportperson",
            name="profile_image",
            field=models.ImageField(
                blank=True,
                max_length=256,
                null=True,
                upload_to=wbcore.contrib.example_app.models.upload_to_profile_images,
            ),
        ),
    ]
