# Generated by Django 4.1.8 on 2023-04-21 08:27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0003_remove_entry_last_event"),
    ]

    operations = [
        migrations.AddField(
            model_name="entry",
            name="is_draft_entry",
            field=models.BooleanField(
                default=False,
                help_text="Draft entries are entries that have been created automatically and still need verification by a human user.",
                verbose_name="Draft Entry",
            ),
        ),
    ]
