# Generated by Django 5.0.8 on 2024-09-09 15:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcore", "0013_delete_colorgradient"),
    ]

    operations = [
        migrations.AddField(
            model_name="biguserobjectpermission",
            name="system",
            field=models.BooleanField(default=False),
        ),
        migrations.SeparateDatabaseAndState(
            database_operations=[],
            state_operations=[
                migrations.AlterUniqueTogether(
                    name="biguserobjectpermission",
                    unique_together=None,
                ),
                migrations.RemoveField(
                    model_name="biguserobjectpermission",
                    name="content_type",
                ),
                migrations.RemoveField(
                    model_name="biguserobjectpermission",
                    name="permission",
                ),
                migrations.RemoveField(
                    model_name="biguserobjectpermission",
                    name="user",
                ),
                migrations.DeleteModel(
                    name="BigGroupObjectPermission",
                ),
                migrations.DeleteModel(
                    name="BigUserObjectPermission",
                ),
            ],
        ),
    ]
