# Generated by Django 4.1.8 on 2023-04-14 13:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="entry",
            name="last_event",
            field=models.ForeignKey(
                blank=True,
                help_text="The last event of this entry. Can be ordered by end date.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="last_event_of",
                to="agenda.calendaritem",
                verbose_name="Last Event",
            ),
        ),
        migrations.RemoveField(
            model_name="company",
            name="asset_allocation",
        ),
        migrations.RemoveField(
            model_name="company",
            name="assets_under_management",
        ),
        migrations.RemoveField(
            model_name="company",
            name="discretionairy_advisory",
        ),
        migrations.RemoveField(
            model_name="company",
            name="geographic_focus",
        ),
    ]
