from .contacts import (
    AddressContactEntryTitleConfig,
    AddressContactTitleConfig,
    BankingContactEntryTitleConfig,
    BankingContactTitleConfig,
    EmailContactEntryTitleConfig,
    EmailContactTitleConfig,
    SocialMediaContactEntryTitleConfig,
    TelephoneContactEntryTitleConfig,
    TelephoneContactTitleConfig,
    WebsiteContactEntryTitleConfig,
)
from .entries import (
    CompanyModelTitleConfig,
    PersonModelTitleConfig,
    UserIsManagerTitleConfig,
)
from .relationships import (
    ClientManagerTitleConfig,
    EmployerEmployeeRelationshipTitleConfig,
    RelationshipEntryModelTitleConfig,
    RelationshipTypeModelTitleConfig,
    UserIsClientTitleConfig,
)
from .utils import (
    CompanyTypeTitleConfig,
    CustomerStatusTitleConfig,
    PositionTitleConfig,
    SpecializationTitleConfig,
)
