# Generated by Django 4.2.6 on 2023-11-16 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcore", "0012_delete_notification"),
    ]
    database_operations = [migrations.AlterModelTable("ColorGradient", "color_colorgradient")]

    state_operations = [migrations.DeleteModel("ColorGradient")]

    operations = state_operations
