from wbcore.frontend_user_configuration import FrontendUserConfiguration
from wbcore.metadata.configs.display.models import AppliedPreset, Preset
from wbcore.release_notes.models import ReleaseNote

from .base import DynamicModel, WBModel
from .fields import DynamicDecimalField, DynamicFloatField, YearField
