{% load i18n %}
{% autoescape off %}
{% blocktranslate trimmed %}Reset Password

You're receiving this email because you requested a password reset for your user account at {{ site_name }}.
Please click and choose a new password{% endblocktranslate %}

{% translate "Reset Password" %}: {{ protocol }}://{{ domain }}{% url 'wbcore:authentication:password_reset_confirm' uidb64=uid token=token %}

{% translate "This E-Mail is autogenerated by Stainly. If you need more information, please visit the Stainly-Bench." %}
{% endautoescape %}
