{% extends 'base.html' %}
{% load i18n %}
{% block content%}

<div class="container-500">
  <h1>{% translate "Password Reset" %}</h1>
</div>
<h1>{% translate "Password reset complete" %}</h1>
<p>{% translate "Your Password has been set. You may go ahead and login." %}</p>
<div class="text-center">
  <a href="{{ login_url }}">{% translate "Login" %}</a>
</div>
{% endblock%}
