# Generated by Django 4.1.9 on 2023-05-24 08:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0002_notificationusertoken_unique_user_token_device"),
    ]

    operations = [
        migrations.AddField(
            model_name="notificationusertoken",
            name="updated",
            field=models.DateTimeField(auto_now=True),
        ),
    ]
