# Generated by Django 5.0.13 on 2025-03-13 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0007_notificationtype_resource_button_label'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationtype',
            name='is_lock',
            field=models.BooleanField(default=False),
        ),
    ]
