{% extends 'base.html' %}
{% load i18n %}
{% block content%}
{% if validlink %}
<div class="container-500">
    <h1>{% translate "Password Reset" %}</h1>
</div>
<h1>{% translate "Password Reset Form" %}</h1>
<p>{% translate "Please enter your new password." %}</p>
<br />
<form style="all: initial;" action="" method="POST">
    {% csrf_token %}
    {{form}}
    <input type="submit" value="Reset Password" />
</form>
{% else %}
<div class="container-500">
    <h1>{% translate "Password Reset" %}</h1>
</div>
<h1>{% translate "Password reset failed" %}</h1>
<p>{% translate "The password reset link was invalid, possibly because it has already been used. Please request a new password reset." %}</p>
{% endif %}
{% endblock%}
